@@ -713,12 +713,7 @@ on_startup (
 	struct pgm_interface_req_t if_req;
 	memset (&if_req, 0, sizeof(if_req));
 	if_req.ir_interface = res->ai_recv_addrs[0].gsr_interface;
-	if_req.ir_scope_id  = 0;
-	if (AF_INET6 == sa_family) {
-		struct sockaddr_in6 sa6;
-		memcpy (&sa6, &res->ai_recv_addrs[0].gsr_group, sizeof(sa6));
-		if_req.ir_scope_id = sa6.sin6_scope_id;
-	}
+	memcpy (&if_req.ir_address, &res->ai_send_addrs[0].gsr_addr, sizeof(struct sockaddr_storage));
 	if (!pgm_bind3 (g_sock,
 			&addr, sizeof(addr),
 			&if_req, sizeof(if_req),	/* tx interface */
@@ -732,7 +727,7 @@ on_startup (
 /* join IP multicast groups */
 	for (unsigned i = 0; i < res->ai_recv_addrs_len; i++)
 	{
-		if (!pgm_setsockopt (g_sock, IPPROTO_PGM, PGM_JOIN_GROUP, &res->ai_recv_addrs[i], sizeof(struct group_req))) {
+		if (!pgm_setsockopt (g_sock, IPPROTO_PGM, PGM_JOIN_GROUP, &res->ai_recv_addrs[i], sizeof(struct pgm_group_source_req))) {
 			char group[INET6_ADDRSTRLEN];
 			getnameinfo ((struct sockaddr*)&res->ai_recv_addrs[i].gsr_group, sizeof(struct sockaddr_in),
                                         group, sizeof(group),
@@ -744,7 +739,7 @@ on_startup (
 			goto err_abort;
 		}
 	}
-	if (!pgm_setsockopt (g_sock, IPPROTO_PGM, PGM_SEND_GROUP, &res->ai_send_addrs[0], sizeof(struct group_req))) {
+	if (!pgm_setsockopt (g_sock, IPPROTO_PGM, PGM_SEND_GROUP, &res->ai_send_addrs[0], sizeof(struct pgm_group_source_req))) {
                 char group[INET6_ADDRSTRLEN];
                 getnameinfo ((struct sockaddr*)&res->ai_send_addrs[0].gsr_group, sizeof(struct sockaddr_in),
 				group, sizeof(group),
@@ -772,10 +767,11 @@ on_startup (
 			g_error ("setting PGM_MULTICAST_HOPS = %d", multicast_hops);
 			goto err_abort;
 		}
+/* ToS only exists for IPv4 */
 		if (AF_INET6 != sa_family) {
+/* Some platforms require additional privilege for setting ToS */
 			if (!pgm_setsockopt (g_sock, IPPROTO_PGM, PGM_TOS, &dscp, sizeof(dscp))) {
-				g_error ("setting PGM_TOS = 0x%x", dscp);
-				goto err_abort;
+				g_warning ("setting PGM_TOS = 0x%x", dscp);
 			}
 		}
 		if (!pgm_setsockopt (g_sock, IPPROTO_PGM, PGM_NOBLOCK, &nonblocking, sizeof(nonblocking))) {
@@ -1278,7 +1274,7 @@ again:
 					break;
 				}
 			}
-/* fall through */
+			/* fallthrough */
 		case PGM_IO_STATUS_WOULD_BLOCK:
 //g_message ("would block");
 block:
