@@ -163,7 +163,7 @@ static pgm_time_t		pgm_rtc_update (void);
 #ifdef HAVE_RDTSC
 #	include <stdio.h>
 #	include <string.h>
-#	if defined(__APPLE__) || defined(__FreeBSD__)
+#	if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
 #		include <sys/sysctl.h>
 #	elif defined(__sun)
 #		include <kstat.h>
@@ -449,6 +449,13 @@ pgm_time_init (
 		if (0 == sysctlbyname ("hw.clockrate", &clockrate, &len, NULL, 0)) {
 			tsc_khz = (uint_fast32_t)(clockrate * 1000);
 		}
+#elif defined(__NetBSD__)
+		uint64_t clockrate;
+		size_t len;
+		len = sizeof (clockrate);
+		if (0 == sysctlbyname ("machdep.tsc_freq", &clockrate, &len, NULL, 0)) {
+			tsc_khz = (uint_fast32_t)(clockrate / 1000);
+		}
 #elif defined(KSTAT_DATA_INT32)
 /* ref: http://developers.sun.com/solaris/articles/kstatc.html */
 		kstat_ctl_t* kc;
@@ -810,11 +817,16 @@ pgm_tsc_init (
  */
 	FILE	*fp = fopen ("/proc/cpuinfo", "r");
 	char	buffer[1024], *flags = NULL;
+	int	tmp_tsc_mhz = 0;
+	bool	ret = FALSE;
 	if (fp)
 	{
 		while (!feof(fp) && fgets (buffer, sizeof(buffer), fp))
 		{
-			if (strstr (buffer, "flags")) {
+			if (strstr (buffer, "cpu MHz")) {
+				const char *p = strchr (buffer, ':');
+				if (p) tmp_tsc_mhz = atoi (p + 1) * 1000;
+			} else if (strstr (buffer, "flags")) {
 				flags = strchr (buffer, ':');
 				break;
 			}
@@ -825,14 +837,18 @@ pgm_tsc_init (
 		pgm_warn (_("Linux kernel reports no Time Stamp Counter (TSC)."));
 /* force both to stable clocks even though one might be OK */
 		pgm_time_update_now	= pgm_gettimeofday_update;
-		return TRUE;
-	}
-	if (!strstr (flags, " constant_tsc")) {
+		ret = TRUE;
+	} else if (!strstr (flags, " constant_tsc")) {
 		pgm_warn (_("Linux kernel reports non-constant Time Stamp Counter (TSC)."));
 /* force both to stable clocks even though one might be OK */
 		pgm_time_update_now	= pgm_gettimeofday_update;
-		return TRUE;
+		ret = TRUE;
 	}
+
+/* Only return in case the kernel doesn't report 0 as cpu MHz */
+	if ((tmp_tsc_mhz != 0) && ret)
+		return TRUE;
+
 #		endif /* HAVE_PROC_CPUINFO */
 
 	pgm_time_t		start, stop, elapsed;
